package ru.dbotthepony.kstarbound.defs.item

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.defs.util.enrollMap
import ru.dbotthepony.kstarbound.io.json.builder.BuilderAdapter
import ru.dbotthepony.kstarbound.io.json.builder.JsonBuilder
import ru.dbotthepony.kstarbound.io.json.builder.JsonIgnoreProperty
import ru.dbotthepony.kstarbound.io.json.util.asJsonObject
import ru.dbotthepony.kstarbound.io.json.util.asList
import ru.dbotthepony.kstarbound.io.json.neverNull
import ru.dbotthepony.kstarbound.util.NotNullVar

@JsonBuilder
open class ArmorItemPrototype : ItemPrototype(), IArmorItemDefinition {
	override var colorOptions: ImmutableList<Map<String, String>> = ImmutableList.of()
	override var maleFrames: IArmorItemDefinition.ArmorFrames by NotNullVar()
	override var femaleFrames: IArmorItemDefinition.ArmorFrames by NotNullVar()
	override var level: Double = 1.0
	override var leveledStatusEffects: ImmutableList<LeveledStatusEffect> = ImmutableList.of()

	override var scripts: ImmutableList<String> = ImmutableList.of()
	override var scriptDelta: Int = 1

	@JsonIgnoreProperty
	override var armorType: ArmorPieceType by NotNullVar()

	init {
		maxStack = 1L
	}

	override fun assemble(): IItemDefinition {
		return ArmorItemDefinition(
			descriptionData = descriptionData,
			itemName = itemName,
			price = price,
			rarity = rarity,
			category = category,
			inventoryIcon = inventoryIcon,
			itemTags = itemTags,
			learnBlueprintsOnPickup = learnBlueprintsOnPickup,
			maxStack = maxStack,
			eventCategory = eventCategory,
			consumeOnPickup = consumeOnPickup,
			pickupQuestTemplates = pickupQuestTemplates,
			scripts = scripts,
			scriptDelta = scriptDelta,
			tooltipKind = tooltipKind,
			twoHanded = twoHanded,
			radioMessagesOnPickup = radioMessagesOnPickup,
			fuelAmount = fuelAmount,

			json = enrollMap(json),

			colorOptions = colorOptions,
			maleFrames = maleFrames,
			femaleFrames = femaleFrames,
			level = level,
			leveledStatusEffects = leveledStatusEffects,

			armorType = armorType,
		)
	}
}

@JsonBuilder
class HeadArmorItemPrototype : ArmorItemPrototype() {
	init {
		armorType = ArmorPieceType.HEAD
	}
}

@JsonBuilder
class ChestArmorItemPrototype : ArmorItemPrototype() {
	init {
		armorType = ArmorPieceType.CHEST
	}
}

@JsonBuilder
class LegsArmorItemPrototype : ArmorItemPrototype() {
	init {
		armorType = ArmorPieceType.LEGS
	}
}

@JsonBuilder
class BackArmorItemPrototype : ArmorItemPrototype() {
	init {
		armorType = ArmorPieceType.BACK
	}
}
