package ru.dbotthepony.kstarbound.defs.item

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kstarbound.defs.util.enrollMap
import ru.dbotthepony.kstarbound.io.json.builder.JsonBuilder
import ru.dbotthepony.kstarbound.util.NotNullVar
import ru.dbotthepony.kvector.vector.ndouble.Vector2d

@JsonBuilder
class HarvestingToolPrototype : ItemPrototype(), IHarvestingToolDefinition {
	override var frames: Int by NotNullVar()
	override var animationCycle: Double by NotNullVar()
	override var blockRadius: Int by NotNullVar()
	override var altBlockRadius: Int = 0
	override var idleSound: ImmutableList<String> = ImmutableList.of()
	override var strikeSounds: ImmutableList<String> = ImmutableList.of()
	override var handPosition: Vector2d by NotNullVar()
	override var fireTime: Double by NotNullVar()

	init {
		maxStack = 1L
	}

	override fun assemble(): IItemDefinition {
		return HarvestingToolDefinition(
			descriptionData = descriptionData,
			itemName = itemName,
			price = price,
			rarity = rarity,
			category = category,
			inventoryIcon = inventoryIcon,
			itemTags = itemTags,
			learnBlueprintsOnPickup = learnBlueprintsOnPickup,
			maxStack = maxStack,
			eventCategory = eventCategory,
			consumeOnPickup = consumeOnPickup,
			pickupQuestTemplates = pickupQuestTemplates,
			tooltipKind = tooltipKind,
			twoHanded = twoHanded,
			radioMessagesOnPickup = radioMessagesOnPickup,
			fuelAmount = fuelAmount,

			json = enrollMap(json),

			frames = frames,
			animationCycle = animationCycle,
			blockRadius = blockRadius,
			altBlockRadius = altBlockRadius,
			idleSound = idleSound,
			strikeSounds = strikeSounds,
			handPosition = handPosition,
			fireTime = fireTime,
		)
	}
}
